Index: packages/compat/posix/current/src/time.cxx
===================================================================
RCS file: /cvs/ecos/ecos/packages/compat/posix/current/src/time.cxx,v
retrieving revision 1.9
diff -u -r1.9 time.cxx
--- packages/compat/posix/current/src/time.cxx	31 Jan 2003 11:53:14 -0000	1.9
+++ packages/compat/posix/current/src/time.cxx	29 Oct 2003 16:59:43 -0000
@@ -105,6 +105,7 @@
     cyg_bool            pending;        // is expiry pending?
     int                 overrun;        // Overrun count
     struct sigevent     sigev;          // Sigevent to raise on expiry
+    pthread_info        *caller;        // Thread object that set the timer
     
     // Space for alarm object
     cyg_uint8           alarm_obj[sizeof(Cyg_Alarm)];
@@ -273,7 +274,23 @@
             sigemptyset( &mask );
             sigaddset( &mask, timer->sigev.sigev_signo );
             cyg_posix_signal_sigwait();
-            cyg_posix_pthread_release_thread( &mask );
+
+            // Wake up a thread to handle the signal.  Give preference
+            // to the thread that initially set the timer.
+            if ((timer->caller != NULL) &&
+                (timer->caller->state <= PTHREAD_STATE_RUNNING) &&
+                ((mask & ~timer->caller->sigmask) != 0))
+            {
+                // The original caller can service at least one of
+                // the pending signals.  Interrupt its current wait
+                // and make its ASR pending.
+                timer->caller->thread->set_asr_pending();
+                timer->caller->thread->release();
+
+            } else {
+                // Find another thread that can handle the signal(s).
+                cyg_posix_pthread_release_thread( &mask );
+            }
         }
         else if( timer->sigev.sigev_notify == SIGEV_THREAD )
         {
@@ -439,6 +456,7 @@
 
     timer->armed        = false;
     timer->overrun      = 0;
+    timer->caller       = NULL;
 
     *timer_id = timer->id;
     
@@ -535,6 +553,9 @@
             // If the ABSTIME flag is not set, add the current time
             if( (flags & TIMER_ABSTIME) == 0 )
                 trigger += Cyg_Clock::real_time_clock->current_value();
+
+            // Record the preferred thread context for signal delivery
+            timer->caller = pthread_self_info();
 
             // Set the alarm running.
             timer->alarm->initialize( trigger, interval );
